@@ -1,6 +1,6 @@
 package Tie::Array::DBD;
 
-our $VERSION = "0.13";
+our $VERSION = "0.14";
 
 use strict;
 use warnings;
@@ -43,7 +43,7 @@ my %DB = (
     SQLite	=> {
 	temp	=> "temporary",
 	t_key	=> "integer not null primary key",
-	t_val	=> "text",
+	t_val	=> "blob",
 	clear	=> "delete from",
 	pbind	=> 0, # TYPEs in SQLite are text, bind_param () needs int
 	autoc	=> 0,
@@ -80,13 +80,13 @@ sub _create_table
 	};
     $exists and return;	# Table already exists
 
-    my ($temp, $t_key, $t_val) = @{$DB{$dbt}}{qw( temp t_key t_val )};
+    my $temp = $DB{$dbt}{temp};
     $cnf->{tmp} or $temp = "";
     local $dbh->{AutoCommit} = 1 unless $dbt eq "CSV" || $dbt eq "Unify";
     $dbh->do (
 	"create $temp table $cnf->{tbl} (".
-	    "$cnf->{f_k} $t_key,".
-	    "$cnf->{f_v} $t_val)"
+	    "$cnf->{f_k} $cnf->{ktp},".
+	    "$cnf->{f_v} $cnf->{vtp})"
 	);
     $dbt eq "Unify" and $dbh->commit;
     } # create table
@@ -123,6 +123,8 @@ sub TIEARRAY
 	tbl => undef,
 	tmp => $tmp,
 	str => undef,
+	ktp => $cnf->{t_key},
+	vtp => $cnf->{t_val},
 	};
 
     if ($opt) {
@@ -132,6 +134,7 @@ sub TIEARRAY
 	$opt->{fld} and $f_v      = $opt->{fld};
 	$opt->{tbl} and $h->{tbl} = $opt->{tbl};
 	$opt->{str} and $h->{str} = $opt->{str};
+	$opt->{vtp} and $h->{vtp} = $opt->{vtp};
 	}
 
     $h->{f_k} = $f_k;
@@ -155,7 +158,7 @@ sub TIEARRAY
     $h->{uky} = $dbh->prepare ("update $tbl set $f_k = ? where $f_k = ?");
 
     unless (exists $cnf->{pbind} && !$cnf->{pbind}) {
-	my $sth = $dbh->prepare ("select $f_k, $f_v from $tbl");
+	my $sth = $dbh->prepare ("select $f_k, $f_v from $tbl where 0 = 1");
 	$sth->execute;
 	my @typ = @{$sth->{TYPE}};
 
@@ -504,14 +507,21 @@ all by itself, but uses the connection provided in the handle.
 If the first argument is a scalar, it is used as DSN for DBI->connect ().
 
 Supported DBD drivers include DBD::Pg, DBD::SQLite, DBD::CSV, DBD::mysql,
-DBD::Oracle, and DBD::Unify.
+DBD::Oracle, DBD::Unify, and DBD::Firebird.  Note that due to limitations
+they won't all perform equally well. Firebird is not tested anymore.
 
 DBD::Pg and DBD::SQLite have an unexpected great performance when server
 is the local system. DBD::SQLite is even almost as fast as DB_File.
 
-The current implementation appears to be extremely slow CSV, as expected,
-mysql, and Unify. For Unify and mysql that is because these do not allow
-indexing on the key field so they cannot be set to be primary key.
+The current implementation appears to be extremely slow for CSV, as
+expected, mysql, and Unify. For Unify and mysql that is because these do
+not allow indexing on the key field so they cannot be set to be primary
+key.
+
+When using DBD::CSV with Text::CSV_XS version 1.02 or newer, it might be
+wise to disable utf8 encoding (only supported as of DBD::CSV-0.48):
+
+ "dbi:CSV:f_ext=.csv/r;csv_null=1;csv_decode_utf8=0"
 
 =head2 Options
 
@@ -542,6 +552,11 @@ C<h_key>.
 Defines the name of the value field in the database table.   The default
 is C<h_value>.
 
+=item vtp
+
+Defines the type of the fld field in the database table.  The default is
+depending on the underlying database and most likely some kind of BLOB.
+
 =item str
 
 Defines the required persistence module. Currently only supports the use
@@ -571,6 +586,22 @@ flags and use the streamer option:
 
   tie my @array, "Tie::Array::DBD", { str => "Storable" };
 
+If you do not want the performance impact of Storable just to be able to
+store and retrieve UTF-8 values, there are two ways to do so:
+
+  # Use utf-8 from database
+  tie my @array, "Tie::Array::DBD", "dbi:Pg:", { vtp => "text" };
+  $array[2] = "The teddybear costs \x{20ac} 45.95";
+
+  # use Encode
+  tie my @array, "Tie::Array::DBD", "dbi:Pg:";
+  $array[2] = encode "UTF-8", "The teddybear costs \x{20ac} 45.95";
+
+Note  that using Encode will allow other binary data too where using the
+database encoding does not:
+
+  $array[2] = pack "L>A*", time, encode "UTF-8", "Price: \x{20ac} 45.95";
+
 =head2 Nesting and deep structures
 
 C<Tie::Array::DBD> stores values as binary data. This means that
@@ -615,6 +646,17 @@ C<CODE>, C<FORMAT>, and C<GLOB>.  Future extensions might implement some
 alternative streaming modules, like C<Data::Dump::Streamer> or use mixin
 approaches that enable you to fit in your own.
 
+=item *
+
+Note that neither DBD::CSV nor DBD::Unify support C<AutoCommit>.
+
+=item *
+
+For now, Firebird does not support C<TEXT> (or C<CLOB>) in DBD::Firebird
+at a level required by Tie::Array::DBD.
+
+Firebird support is stalled.
+
 =back
 
 =head1 TODO
@@ -653,7 +695,7 @@ H.Merijn Brand <h.m.brand@xs4all.nl>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2010-2014 H.Merijn Brand
+Copyright (C) 2010-2015 H.Merijn Brand
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
