@@ -1,6 +1,6 @@
 package Tie::Hash::DBD;
 
-our $VERSION = "0.13";
+our $VERSION = "0.14";
 
 use strict;
 use warnings;
@@ -49,7 +49,7 @@ my %DB = (
     SQLite	=> {
 	temp	=> "temporary",
 	t_key	=> "text primary key",
-	t_val	=> "text",
+	t_val	=> "blob",
 	clear	=> "delete from",
 	pbind	=> 0, # TYPEs in SQLite are text, bind_param () needs int
 	autoc	=> 0,
@@ -86,13 +86,13 @@ sub _create_table
 	};
     $exists and return;	# Table already exists
 
-    my ($temp, $t_key, $t_val) = @{$DB{$dbt}}{qw( temp t_key t_val )};
+    my $temp = $DB{$dbt}{temp};
     $cnf->{tmp} or $temp = "";
     local $dbh->{AutoCommit} = 1 unless $dbt eq "CSV" || $dbt eq "Unify";
     $dbh->do (
 	"create $temp table $cnf->{tbl} (".
-	    "$cnf->{f_k} $t_key,".
-	    "$cnf->{f_v} $t_val)"
+	    "$cnf->{f_k} $cnf->{ktp},".
+	    "$cnf->{f_v} $cnf->{vtp})"
 	);
     $dbt eq "Unify" and $dbh->commit;
     } # create table
@@ -131,6 +131,8 @@ sub TIEHASH
 	str => undef,
 	asc => $cnf->{k_asc} || 0,
 	trh => 0,
+	ktp => $cnf->{t_key},
+	vtp => $cnf->{t_val},
 	};
 
     if ($opt) {
@@ -141,6 +143,8 @@ sub TIEHASH
 	$opt->{tbl} and $h->{tbl} = $opt->{tbl};
 	$opt->{str} and $h->{str} = $opt->{str};
 	$opt->{trh} and $h->{trh} = $opt->{trh};
+	$opt->{ktp} and $h->{ktp} = $opt->{ktp};
+	$opt->{vtp} and $h->{vtp} = $opt->{vtp};
 	}
 
     $h->{f_k} = $f_k;
@@ -166,7 +170,7 @@ sub TIEHASH
     $h->{ctv} = $dbh->prepare ("select count(*) from $tbl where $f_k = ?");
 
     unless (exists $cnf->{pbind} && !$cnf->{pbind}) {
-	my $sth = $dbh->prepare ("select $f_k, $f_v from $tbl");
+	my $sth = $dbh->prepare ("select $f_k, $f_v from $tbl where 0 = 1");
 	$sth->execute;
 	my @typ = @{$sth->{TYPE}};
 
@@ -207,7 +211,7 @@ sub STORE
     my ($self, $key, $value) = @_;
     my $k = $self->{asc} ? unpack "H*", $key : $key;
     my $v = $self->_stream ($value);
-    $self->{trh} and $self->{dbh}->begin_work;
+    $self->{trh} and $self->{dbh}->begin_work unless $self->{dbt} eq "SQLite";
     my $r = $self->EXISTS ($key)
 	? $self->{upd}->execute ($v, $k)
 	: $self->{ins}->execute ($k, $v);
@@ -219,7 +223,7 @@ sub DELETE
 {
     my ($self, $key) = @_;
     $self->{asc} and $key = unpack "H*", $key;
-    $self->{trh} and $self->{dbh}->begin_work;
+    $self->{trh} and $self->{dbh}->begin_work unless $self->{dbt} eq "SQLite";
     $self->{sel}->execute ($key);
     my $r = $self->{sel}->fetch;
     unless ($r) {
@@ -258,7 +262,7 @@ sub FETCH
 sub FIRSTKEY
 {
     my $self = shift;
-    $self->{trh} and $self->{dbh}->begin_work;
+    $self->{trh} and $self->{dbh}->begin_work unless $self->{dbt} eq "SQLite";
     $self->{key} = $self->{dbh}->selectcol_arrayref ("select $self->{f_k} from $self->{tbl}");
     $self->{trh} and $self->{dbh}->commit;
     unless (@{$self->{key}}) {
@@ -379,14 +383,20 @@ If the first argument is a scalar, it is used as DSN for DBI->connect ().
 
 Supported DBD drivers include DBD::Pg, DBD::SQLite, DBD::CSV, DBD::mysql,
 DBD::Oracle, DBD::Unify, and DBD::Firebird.  Note that due to limitations
-they won't all perform equally well.
+they won't all perform equally well. Firebird is not tested anymore.
 
 DBD::Pg and DBD::SQLite have an unexpected great performance when server
 is the local system. DBD::SQLite is even almost as fast as DB_File.
 
-The current implementation appears to be extremely slow CSV, as expected,
-mysql, and Unify. For Unify and mysql that is because these do not allow
-indexing on the key field so they cannot be set to be primary key.
+The current implementation appears to be extremely slow for CSV, as
+expected, mysql, and Unify. For Unify and mysql that is because these do
+not allow indexing on the key field so they cannot be set to be primary
+key.
+
+When using DBD::CSV with Text::CSV_XS version 1.02 or newer, it might be
+wise to disable utf8 encoding (only supported as of DBD::CSV-0.48):
+
+ "dbi:CSV:f_ext=.csv/r;csv_null=1;csv_decode_utf8=0"
 
 =head2 Options
 
@@ -415,11 +425,21 @@ unless you provide a true C<trh> attribute.
 Defines the name of the key field in the database table.  The default is
 C<h_key>.
 
+=item ktp
+
+Defines the type of the key field in the database table.  The default is
+depending on the underlying database. Probably unwise to change.
+
 =item fld
 
 Defines the name of the value field in the database table.   The default
 is C<h_value>.
 
+=item vtp
+
+Defines the type of the fld field in the database table.  The default is
+depending on the underlying database and most likely some kind of BLOB.
+
 =item str
 
 Defines the required persistence module. Currently only supports the use
@@ -456,6 +476,22 @@ flags and use the streamer option:
 
   tie my %hash, "Tie::Hash::DBD", { str => "Storable" };
 
+If you do not want the performance impact of Storable just to be able to
+store and retrieve UTF-8 values, there are two ways to do so:
+
+  # Use utf-8 from database
+  tie my %hash, "Tie::Hash::DBD", "dbi:Pg:", { vtp => "text" };
+  $hash{foo} = "The teddybear costs \x{20ac} 45.95";
+
+  # use Encode
+  tie my %hash, "Tie::Hash::DBD", "dbi:Pg:";
+  $hash{foo} = encode "UTF-8", "The teddybear costs \x{20ac} 45.95";
+
+Note  that using Encode will allow other binary data too where using the
+database encoding does not:
+
+  $hash{foo} = pack "L>A*", time, encode "UTF-8", "Price: \x{20ac} 45.95";
+
 =head2 Nesting and deep structures
 
 C<Tie::Hash::DBD> stores keys and values as binary data. This means that
@@ -515,7 +551,9 @@ Note that neither DBD::CSV nor DBD::Unify support C<AutoCommit>.
 For now, Firebird does not support C<TEXT> (or C<CLOB>) in DBD::Firebird
 at a level required by Tie::Hash::DBD. Neither does it support arbitrary
 length index on C<VARCHAR> fields so it can neither be a primary key nor
-can it be the subject of a (unique) index, hence large sets will be slow
+can it be the subject of a (unique) index hence large sets will be slow.
+
+Firebird support is stalled.
 
 =back
 
@@ -555,7 +593,7 @@ H.Merijn Brand <h.m.brand@xs4all.nl>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2010-2014 H.Merijn Brand
+Copyright (C) 2010-2015 H.Merijn Brand
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
